#[non_exhaustive]pub enum BmpDecoderErrors {
InvalidMagicBytes,
TooSmallBuffer(usize, usize),
GenericStatic(&'static str),
Generic(String),
TooLargeDimensions(&'static str, usize, usize),
OverFlowOccurred,
IoErrors(ZByteIoError),
}Expand description
BMP errors that can occur during decoding
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
InvalidMagicBytes
The file/bytes do not start with BM
TooSmallBuffer(usize, usize)
The output buffer is too small, expected at least a size but got another size
GenericStatic(&'static str)
Generic message
Generic(String)
Generic allocated message
TooLargeDimensions(&'static str, usize, usize)
Too large dimensions for a given width or height
OverFlowOccurred
A calculation overflowed
IoErrors(ZByteIoError)