#[non_exhaustive]pub enum ImageFormat {
JPEG,
PNG,
PPM,
PSD,
Farbfeld,
QOI,
JPEG_XL,
HDR,
BMP,
WEBP,
Unknown,
}Expand description
All supported image formats
This enum contains supported image formats, either encoders or decoders for a particular image
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
JPEG
Joint Photographic Experts Group
PNG
Portable Network Graphics
PPM
Portable Pixel Map image
PSD
Photoshop PSD component
Farbfeld
Farbfeld format
QOI
Quite Okay Image
JPEG_XL
JPEG XL, new format
HDR
Radiance HDR decoder
BMP
Windows Bitmap Files
WEBP
Unknown
Any unknown format
Implementations
Sourceimpl ImageFormat
impl ImageFormat
pub fn has_decoder(self) -> bool
pub fn decoder<'a, T>( &self, data: T, ) -> Result<Box<dyn DecoderTrait + 'a>, ImageErrors>
pub fn decoder_with_options<'a, T>( &self, data: T, options: DecoderOptions, ) -> Result<Box<dyn DecoderTrait + 'a>, ImageErrors>
Sourcepub fn has_encoder(&self) -> bool
pub fn has_encoder(&self) -> bool
Return true if an image format has an encoder that can convert the image into that format
pub fn encode<T: ZByteWriterTrait>( &self, image: &Image, encoder_options: EncoderOptions, sink: T, ) -> Result<usize, ImageErrors>
pub fn guess_format<T>(bytes: T) -> Option<(ImageFormat, T)>where
T: ZByteReaderTrait,
pub fn encoder_for_extension<P: AsRef<str>>(extension: P) -> Option<ImageFormat>