pub enum HdrEncodeErrors {
UnsupportedColorspace(ColorSpace),
WrongInputSize(usize, usize),
Static(&'static str),
IoErrors(ZByteIoError),
}Expand description
HDR encoding errrors
Variants
UnsupportedColorspace(ColorSpace)
The colorspace provided by user is not supported by HDR
WrongInputSize(usize, usize)
The input size was expected to be of a certain size but isn’t
Static(&'static str)
Generic message
IoErrors(ZByteIoError)