ImageMetadata

Struct ImageMetadata 

pub struct ImageMetadata { /* private fields */ }
Image metadata

Each image type has this information present The decoder usually sets this up while the encoder can get these details from the user/image struct

Implementations

Source

impl ImageMetadata

pub fn parse_raw_exif(&mut self, data: &[u8])

Parse raw Exif and store it as a field in the data

Data should point to the first exif byte

This requires the metadata feature to be activated otherwise it’s a compile error

Source

impl ImageMetadata

pub const fn exif(&self) -> Option<&Vec<Field>>

Return the exif metadata of an image or none if it doesn’t exist

pub fn exif_mut(&mut self) -> Option<&mut Vec<Field>>

Return a mutable reference to the exif metadata of an image or none if it doesn’t exist

pub const fn dimensions(&self) -> (usize, usize)

Get image dimensions as a tuple of width and height

Example
use zune_image::metadata::ImageMetadata;
let meta = ImageMetadata::default();
// default dimensions are usually zero
assert_eq!(meta.dimensions(),(0,0));
pub fn set_dimensions(&mut self, width: usize, height: usize)

Set image dimensions

Example
use zune_image::metadata::ImageMetadata;
let mut meta = ImageMetadata::default();
// set image dimensions
meta.set_dimensions(23,24);
// get image dimensions
assert_eq!(meta.dimensions(),(23,24));
pub const fn colorspace(&self) -> ColorSpace

Get an image’s colorspace

The default colorspace is usually ColorSpace::Unknown which represents an uninitialized image

pub fn set_colorspace(&mut self, colorspace: ColorSpace)

Set the image’s colorspace

pub const fn color_trc(&self) -> Option<ColorCharacteristics>

Get color transfer characteristics

Color transfer characteristics tell us more about how the colorspace values are represented whether they are linear or gamma encoded

pub fn set_color_trc(&mut self, trc: ColorCharacteristics)

Set color transfer characteristics for this image

pub const fn depth(&self) -> BitDepth

Get the image bit depth

Default value is BitDepth::Unknown which indicates that the bit-depth is currently unknown for a particular image

pub fn set_depth(&mut self, depth: BitDepth)

Set the image bit depth

pub fn set_default_gamma(&mut self, gamma: f32)

Set the default gamma for this image

This is gamma that will be used to convert this image from gamma colorspace to linear colorspace and back.

Do not set this in between operations, and do not set this if you do not know what you are doing.

The library will set this automatically for supported decoders (those which specify gamma during transfer)

Arguments
  • gamma : The new gamma value
pub const fn image_format(&self) -> Option<ImageFormat>

Get the image for which this metadata was fetched from

May be None if the caller didn’t set a format

pub const fn alpha(&self) -> AlphaState

pub fn is_premultiplied_alpha(&self) -> bool

pub fn set_alpha(&mut self, alpha_state: AlphaState)

pub fn set_icc_chunk(&mut self, icc_chunk: Vec<u8>)

pub fn icc_chunk(&self) -> Option<&Vec<u8>>

Return the icc chunk of the image

pub fn is_linear(&self) -> bool

Return whether the image is in linear colorspace or not

if true -> Image in linear colorspace if false -> Image in gamma colorspace

pub fn set_linear(&mut self, linear: bool)

Set whether the image is in linear colorspace or not

if true -> Image in linear colorspace if false -> Image in gamma colorspace