pub struct EncodeResult { /* private fields */ }Expand description
A struct holding the result of an encode operation
It contains the image format the data is in
Implementations
Sourceimpl EncodeResult
impl EncodeResult